import sys
import os
import re
import unittest

import psycopg2

import tempus
from tempus import Cost, Request
from datetime import datetime

g_db_options = os.getenv('TEMPUS_DB_OPTIONS', 'dbname=tempus_test_db')

def road_node_id_from_coordinates(cur, pt_xy):
    cur.execute("select tempus.road_node_id_from_coordinates(%s,%s)", pt_xy)
    return cur.fetchone()[0]

if __name__ == '__main__':

    # initialize the tempus library (mandatory)
    tempus.init()

    # load a routing plugin
    plugin = tempus.load_plugin({'db/options': g_db_options}, plugin_name="isochrone_plugin")

    # keep a connection to the db
    conn = psycopg2.connect(g_db_options)
    cursor = conn.cursor()


    m = {}

    # call the plugin
    limit = 20.0
    results = tempus.request(plugin,
                             origin = road_node_id_from_coordinates(cursor, (-1.54427674939367, 47.2010310270336)),
                             steps = [ Request.Step(constraint = Request.TimeConstraint(
                                 type = Request.TimeConstraintType.ConstraintAfter,
                                 date_time = datetime(2016,10,21,6,43))) ],
                             plugin_options = { 'Isochrone/limit' : limit },
                             criteria = [Cost.Duration],
                             # pedestrian + TRAM
                             allowed_transport_modes = [1, 5])

    # explore the resulting isochrone
    vertices = {x.uid: x for x in results[0].isochrone()}
    def browse(x):
        print(x.uid, (x.predecessor, x.mode, x.cost, x.wait_time))
        total_wait_time = 0.0
        cost_per_mode = {}
        if x.predecessor != x.uid:
            cost_per_mode, total_wait_time = browse(vertices[x.predecessor])
        if x.mode not in cost_per_mode:
            cost_per_mode[x.mode] = 0.0
        total_wait_time += x.wait_time
        cost_per_mode[x.mode] += x.cost
        return cost_per_mode, total_wait_time

    v = 130
    cost_per_mode, total_wait_time = browse(vertices[v])
    print("Waiting time: {:.1f} mins".format(total_wait_time))
    print("Total cost: {:.1f} mins".format(sum(cost_per_mode.values())))
    print("Accumulated costs per mode:")
    print("\n".join("{}: {:.1f} mins".format(k,v) for k,v in cost_per_mode.items()))
    #print("\n".join(["{},{},{},{}".format(x.uid, x.predecessor, x.x, x.y) for x in results[0].isochrone()]))



